==> adeptgenerator-8.0.0%2B1.tar.gz: lib/src/diff_generator.dart <==
import 'dart:async';

import 'package:adeptannotations/adeptannotations.dart';
import 'package:adeptgenerator/src/base_generator_for_annotation.dart';
import 'package:adeptgenerator/src/config/diff_key_config.dart';
import 'package:analyzer/dart/constant/value.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';

class DiffGenerator
    extends BaseGeneratorForAnnotation<Differentiable, DiffKey> {
  @override
  FutureOr<String> generateForAnnotatedElement(
      Element element, ConstantReader annotation, BuildStep buildStep) {
    ClassElement classElement =
        getClassElement(element, annotationName: '@Differentiable()');
    String diffFunction = _generateDiffFunction(classElement, annotation);
    return '''
    $diffFunction
    ''';
  }

  /// This function manages the logic of generating the Diff boilerplate code
  String _generateDiffFunction(
      ClassElement classElement, ConstantReader annotation) {
    checkDefaultConstructor(classElement, annotationName: '@Differentiable()');
    String classType = classElement.displayName;
    String functionName = '\$${classType}Diff';
    String currentDataParam = 'currentData';
    String newDataParam = 'newData';
    String functionParams = '{$classType? $currentDataParam, '
        '$classType? $newDataParam}';
    String diffVar = 'diff';
    String isDiffVar = 'isDiff';

    final diffData = StringBuffer('final $diffVar = $classType()');
    final isDiffData = StringBuffer('');
    final differentiable = _getDifferentiable(classElement) ?? const Differentiable();

    void fieldAnalyzer(FieldElement field) {
      if (field.isSynthetic && differentiable.ignoreSynthetic) return;
      if (field.isFinal || field.isConst || field.isStatic) return;
      if (field.getter == null || field.setter == null) return;
      final diffKey = _getDiffKey(field) ?? const DiffKeyConfig();
      if (diffKey.ignore) return;

      final fieldName = field.displayName;
      String currentData = '$currentDataParam.$fieldName';
      String newData = '$newDataParam.$fieldName';
      String diffDataFieldValue = '$newData != '
          "$currentData ? $newData : ${diffKey.defaultValue != null ? "${diffKey.defaultValue}" : currentData}";

      if (diffKey.forceCurrentData) {
        diffDataFieldValue = '$currentDataParam.$fieldName';
      } else if (diffKey.forceNewData) {
        diffDataFieldValue = '$newDataParam.$fieldName';
      } else if (diffKey.diffFunction != null) {
        diffDataFieldValue = diffKey.diffFunction!;
      }

      diffData.writeln('..$fieldName = $diffDataFieldValue');
      isDiffData.writeln(
          '$isDiffVar |= $diffVar.$fieldName != $currentDataParam.$fieldName;');
    }

    ///Recursive "postOrder" visitor to ensure fields analysis in a inheritance logical order
    void superTypeVisitor(InterfaceType? iType) {
      if (iType == null || iType.isDartCoreObject) return;
      superTypeVisitor(iType.element.supertype);
      iType.element.fields.forEach(fieldAnalyzer);
    }

    superTypeVisitor(classElement.supertype);
    classElement.fields.forEach(fieldAnalyzer);
    diffData.writeln(';');

    return '''
      /// This function creates a new instance of [$classType] with only the 
      /// fields that are not equal (==) between [$currentDataParam] and 
      /// [$newDataParam] 
      $classType? $functionName($functionParams){
        
        if(newData == null) return currentData;
        if(currentData == null) return newData;
        
        ${diffData.toString()}
        
        bool $isDiffVar = false;
        ${isDiffData.toString()}
        
        return $isDiffVar ? $diffVar : null;
      }
    ''';
  }

  /// Gets the Differentiable annotation for a given ClassElement
  Differentiable? _getDifferentiable(ClassElement classElement) {
    DartObject? differentiableAnnotation = annotationChecker
        .firstAnnotationOf(classElement, throwOnUnresolved: false);
    if (differentiableAnnotation != null) {
      return Differentiable(
          ignoreSynthetic: differentiableAnnotation
              .getField('ignoreSynthetic')!
              .toBoolValue()!);
    }
    return null;
  }

  /// Gets the DiffKey annotation for a given FieldElement
  DiffKeyConfig? _getDiffKey(FieldElement field) {
    Iterable<DartObject> annotations =
        keyAnnotationChecker.annotationsOfExact(field);
    int annotationsCount = annotations.length;
    if (annotationsCount > 1) {
      throw InvalidGenerationSourceError(
          'Repeated `@DiffKey` annotation on ${field.displayName}.',
          todo: 'Check the `@DiffKey` annotations on field '
              '${field.displayName}.',
          element: field);
    }
    if (annotationsCount == 1) {
      DartObject annotation = annotations.first;

      String? diffFunctionDefinition;
      final function = annotation.getField('diffFunction')?.toFunctionValue();
      if (function != null) {
        String enclosingClass = function.enclosingElement.displayName;
        String functionName = function.displayName;
        String functionParams =
            'currentData: currentData.${field.displayName}, newData: newData.${field.displayName}';
        diffFunctionDefinition =
            '$enclosingClass.$functionName($functionParams)';
      }

      Object? defaultValue = annotationValue(annotation, field, 'defaultValue');
      final key = DiffKeyConfig(
        ignore: annotation.getField('ignore')?.toBoolValue(),
        forceCurrentData:
            annotation.getField('forceCurrentData')?.toBoolValue(),
        forceNewData: annotation.getField('forceNewData')?.toBoolValue(),
        defaultValue: defaultValue,
        diffFunction: diffFunctionDefinition,
      );
      if (key.forceCurrentData && key.forceNewData) {
        throw InvalidGenerationSourceError(
            'forceCurrentData and forceNewData properties on '
            '${field.displayName} are mutually exclusive and can\'t be true '
            'on the same field.',
            todo: 'Check the `@DiffKey` properties on field '
                '${field.displayName}.',
            element: field);
      }
      return key;
    }
    return null;
  }
}

==> adeptgenerator-9.0.0%2B1.tar.gz: lib/src/diff_generator.dart <==
import 'dart:async';

import 'package:adeptannotations/adeptannotations.dart';
import 'package:adeptgenerator/src/base_generator_for_annotation.dart';
import 'package:adeptgenerator/src/config/diff_key_config.dart';
import 'package:analyzer/dart/constant/value.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';

class DiffGenerator
    extends BaseGeneratorForAnnotation<Differentiable, DiffKey> {
  final Differentiable globalAnnotation;
  final bool defaultIgnoreSynthetic = false;
  final bool defaultGenerateExtension = true;

  DiffGenerator(BuilderOptions options)
      : globalAnnotation = Differentiable(
          ignoreSynthetic: options.config['diff_ignore_synthetic'],
          generateDiffExtension: options.config['diff_generate_diff_extension'],
        );

  @override
  FutureOr<String> generateForAnnotatedElement(
      Element element, ConstantReader annotation, BuildStep buildStep) {
    ClassElement classElement =
        getClassElement(element, annotationName: '@Differentiable()');
    String diffFunction = _generateDiffFunction(classElement, annotation);
    return '''
    $diffFunction
    ''';
  }

  /// This function manages the logic of generating the Diff boilerplate code
  String _generateDiffFunction(
      ClassElement classElement, ConstantReader annotation) {
    checkDefaultConstructor(classElement, annotationName: '@Differentiable()');
    String classType = classElement.displayName;
    String functionName = '\$${classType}Diff';
    String currentDataParam = 'currentData';
    String newDataParam = 'newData';
    String functionParams = '{$classType? $currentDataParam, '
        '$classType? $newDataParam}';
    String diffVar = 'diff';
    String isDiffVar = 'isDiff';

    final diffData = StringBuffer('final $diffVar = $classType()');
    final isDiffData = StringBuffer('');
    final differentiableAnnotation =
        _getDifferentiable(classElement) ?? const Differentiable();

    void fieldAnalyzer(FieldElement field) {
      final diffKey = _getDiffKey(field) ?? const DiffKeyConfig();
      final ignoreSynthetic = diffKey.ignoreSynthetic ??
          differentiableAnnotation.ignoreSynthetic ??
          globalAnnotation.ignoreSynthetic ??
          defaultIgnoreSynthetic;

      if (field.isSynthetic && ignoreSynthetic) return;
      if (field.isFinal || field.isConst || field.isStatic) return;
      if (field.getter == null || field.setter == null) return;
      if (diffKey.ignore) return;

      final fieldName = field.displayName;
      String currentData = '$currentDataParam.$fieldName';
      String newData = '$newDataParam.$fieldName';
      String diffDataFieldValue = '$newData != '
          "$currentData ? $newData : ${diffKey.defaultValue != null ? "${diffKey.defaultValue}" : currentData}";

      if (diffKey.forceCurrentData) {
        diffDataFieldValue = '$currentDataParam.$fieldName';
      } else if (diffKey.forceNewData) {
        diffDataFieldValue = '$newDataParam.$fieldName';
      } else if (diffKey.diffFunction != null) {
        diffDataFieldValue = diffKey.diffFunction!;
      }

      diffData.writeln('..$fieldName = $diffDataFieldValue');
      isDiffData.writeln(
          '$isDiffVar |= $diffVar.$fieldName != $currentDataParam.$fieldName;');
    }

    superTypeVisitor(classElement.supertype, fieldAnalyzer);
    classElement.fields.forEach(fieldAnalyzer);
    diffData.writeln(';');

    final generateExtension = differentiableAnnotation.generateDiffExtension ??
        globalAnnotation.generateDiffExtension ??
        defaultGenerateExtension;

    return '''
      /// This function creates a new instance of [$classType] with only the 
      /// fields that are not equal (==) between [$currentDataParam] and 
      /// [$newDataParam] 
      $classType? $functionName($functionParams){
        
        if(newData == null) return currentData;
        if(currentData == null) return newData;
        
        ${diffData.toString()}
        
        bool $isDiffVar = false;
        ${isDiffData.toString()}
        
        return $isDiffVar ? $diffVar : null;
      }
      
      ${!generateExtension ? '' : '''
      extension \$${classType}DiffExtension on $classType {
          $classType? diff({$classType? data}) {
          return $functionName(currentData: this, newData: data);
        }
      }
      '''}
    ''';
  }

  /// Gets the Differentiable annotation for a given ClassElement
  Differentiable? _getDifferentiable(ClassElement classElement) {
    DartObject? differentiableAnnotation = annotationChecker
        .firstAnnotationOf(classElement, throwOnUnresolved: false);
    if (differentiableAnnotation != null) {
      return Differentiable(
        ignoreSynthetic:
            differentiableAnnotation.getField('ignoreSynthetic')?.toBoolValue(),
        generateDiffExtension: differentiableAnnotation
            .getField('generateDiffExtension')
            ?.toBoolValue(),
      );
    }
    return null;
  }

  /// Gets the DiffKey annotation for a given FieldElement
  DiffKeyConfig? _getDiffKey(FieldElement field) {
    Iterable<DartObject> annotations =
        keyAnnotationChecker.annotationsOfExact(field);
    int annotationsCount = annotations.length;
    if (annotationsCount > 1) {
      throw InvalidGenerationSourceError(
          'Repeated `@DiffKey` annotation on ${field.displayName}.',
          todo: 'Check the `@DiffKey` annotations on field '
              '${field.displayName}.',
          element: field);
    }
    if (annotationsCount == 1) {
      DartObject annotation = annotations.first;

      String? diffFunctionDefinition;
      final function = annotation.getField('diffFunction')?.toFunctionValue();
      if (function != null) {
        String enclosingClass = function.enclosingElement.displayName;
        String functionName = function.displayName;
        String functionParams =
            'currentData: currentData.${field.displayName}, newData: newData.${field.displayName}';
        diffFunctionDefinition =
            '$enclosingClass.$functionName($functionParams)';
      }

      Object? defaultValue = annotationValue(annotation, field, 'defaultValue');
      final key = DiffKeyConfig(
        ignore: annotation.getField('ignore')?.toBoolValue(),
        ignoreSynthetic: annotation.getField('ignoreSynthetic')?.toBoolValue(),
        forceCurrentData:
            annotation.getField('forceCurrentData')?.toBoolValue(),
        forceNewData: annotation.getField('forceNewData')?.toBoolValue(),
        defaultValue: defaultValue,
        diffFunction: diffFunctionDefinition,
      );
      if (key.forceCurrentData && key.forceNewData) {
        throw InvalidGenerationSourceError(
            'forceCurrentData and forceNewData properties on '
            '${field.displayName} are mutually exclusive and can\'t be true '
            'on the same field.',
            todo: 'Check the `@DiffKey` properties on field '
                '${field.displayName}.',
            element: field);
      }
      return key;
    }
    return null;
  }
}
